// $Id: Application.cpp,v 1.35 2014/01/15 14:01:52 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Application.h"
#include "Application.moc"

#include "BaseIcons.h"
#include "BaseStatusBar.h"
#include "Client.h"
#include "ConfigurationDialog.h"
#include "Debug.h"
#include "File.h"
#include "IconEngine.h"
#include "XmlOptions.h"
#include "QtUtil.h"
#include "ServerCommand.h"
#include "MainWindow.h"
#include "TreeView.h"

//___________________________________________
Application::Application( CommandLineArguments arguments ):
    BaseApplication( 0, arguments ),
    Counter( "Application" ),
    mainWindow_( 0 )
{}

//___________________________________________
Application::~Application( void )
{}

//___________________________________________
bool Application::realizeWidget( void )
{

    Debug::Throw( "Application::RealizeWidget.\n" );
    if( !BaseApplication::realizeWidget() ) return false;

    mainWindow_ = new MainWindow( 0 );
    mainWindow().setWindowTitle( "Serverlog" );

    connect( &mainWindow(),
        SIGNAL(requestCommand(SERVER::ServerCommand::CommandType)),
        SLOT(_sendRequest(SERVER::ServerCommand::CommandType)) );

    connect( &mainWindow(),
        SIGNAL(requestCommand(SERVER::ServerCommand)),
        SLOT(_sendRequest(SERVER::ServerCommand)) );

    connect(
        &_applicationManager(), SIGNAL(initialized()),
        &mainWindow().resetAction(), SLOT(trigger()) );

    // update configuration
    emit configurationChanged();

    // send first request (to identify all applications)
    _requestIdentify();

    // show main widget
    mainWindow().centerOnDesktop();
    mainWindow().show();

    return true;

}

//___________________________________________
bool Application::initApplicationManager( void )
{

    Debug::Throw( "Application::initApplicationManager.\n" );

    // make sure application manager does not already exist
    if( _hasApplicationManager() ) return true;

    // parse arguments and forbid --no-server options
    CommandLineParser parser( commandLineParser( _arguments() ) );
    if( parser.hasFlag( "--no-server" ) )
    {

        Debug::Throw(0) << "serverlog - by definition the application cannot work in 'no-server' mode." << endl;
        Debug::Throw(0) << "Aborting." << endl;
        return false;

    } else if( !BaseApplication::initApplicationManager() ) return false;
    else {

        connect( &_applicationManager().client(), SIGNAL(commandAvailable(SERVER::ServerCommand)), SLOT(_append(SERVER::ServerCommand)) );
        return true;

    }
}


//____________________________________________
void Application::usage( void ) const
{
    _usage( "serverlog" );
    commandLineParser().usage();
    return;
}

//_______________________________________________
void Application::_configuration( void )
{

    Debug::Throw( "Application::_configuration.\n" );

    // save configuration
    emit saveConfiguration();

    ConfigurationDialog dialog(0);
    connect( &dialog, SIGNAL(configurationChanged()), SIGNAL(configurationChanged()) );
    dialog.centerOnWidget( qApp->activeWindow() );
    dialog.exec();

}

//__________________________________________________________________
void Application::_sendRequest( SERVER::ServerCommand command )
{
    Debug::Throw( "Application::_sendRequest.\n" );
    _applicationManager().client().sendCommand( command );
    return;
}

//___________________________________________
void Application::_append( SERVER::ServerCommand command )
{

    Debug::Throw( "Application::_append.\n" );

    if( mainWindow_ )
    { mainWindow().append( ServerCommandModel::List() << command ); }

}

//________________________________________________
bool Application::_processCommand( SERVER::ServerCommand command )
{

    Debug::Throw( "Application::_processCommand.\n" );
    if( BaseApplication::_processCommand( command ) ) return true;
    if( command.command() == SERVER::ServerCommand::Raise )
    {
        if( mainWindow_ ) mainWindow().uniconify();
        return true;
    } else return false;

}
