#ifndef Application_h
#define Application_h

// $Id: Application.h,v 1.22 2014/01/12 18:36:25 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "BaseApplication.h"
#include "CommandLineArguments.h"
#include "Config.h"
#include "Counter.h"
#include "IconEngine.h"

#include <QModelIndex>

class MainWindow;

//! display application manager server messages
class Application: public BaseApplication, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    Application( CommandLineArguments );

    //! destructor
    ~Application( void );

    //! initialize application manager
    bool initApplicationManager( void );

    //! create all widgets
    bool realizeWidget( void );

    //! command list
    MainWindow& mainWindow( void )
    { return *mainWindow_; }

    //!@name application information
    //@{

    //! usage
    void usage( void ) const;

    //! application name
    virtual QString applicationName( void ) const
    { return "Serverlog"; }

    //! application icon
    virtual QIcon applicationIcon( void ) const
    { return IconEngine::get( ":/serverlog.png" ); }

    // application version
    virtual QString applicationVersion( void ) const
    { return VERSION; }

    // build time stamp
    virtual QString buildTimeStamp( void ) const
    { return BUILD_TIMESTAMP; }

    //@}

    protected Q_SLOTS:

    //! configuration
    void _configuration( void );

    //! pass command to application manager
    void _sendRequest( SERVER::ServerCommand::CommandType type )
    { if( _hasApplicationManager() ) _sendRequest( SERVER::ServerCommand( _applicationManager().id(), type ) ); }

    //! request identification
    void _requestIdentify( void )
    { _sendRequest( SERVER::ServerCommand::Identify ); }

    //! pass command to application manager
    void _sendRequest( SERVER::ServerCommand );

    //! append message to the log
    void _append( SERVER::ServerCommand );

    //! process request from application manager
    virtual bool _processCommand( SERVER::ServerCommand );

    private:

    //! main window
    MainWindow* mainWindow_;

};

#endif
