// $Id: ConfigurationDialog.cpp,v 1.37 2014/01/10 12:14:16 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/
#include "ConfigurationDialog.h"
#include "ConfigurationDialog.moc"

#include "Debug.h"
#include "GridLayout.h"
#include "IconEngine.h"
#include "BaseIcons.h"
#include "OptionCheckBox.h"
#include "OptionComboBox.h"
#include "OptionColorDisplay.h"
#include "Options.h"
#include "QtUtil.h"
#include "ServerConfiguration.h"

#include <QLabel>
#include <QGroupBox>
#include <QLayout>

//_________________________________________________________
ConfigurationDialog::ConfigurationDialog( QWidget* parent ):
BaseConfigurationDialog( parent )
{


    Debug::Throw( "ConfigurationDialog::ConfigurationDialog.\n" );
    setWindowTitle( "Configuration - Serverlog" );

    // base configuration
    baseConfiguration( 0 );

    // generic objects
    QWidget *box;

    // appearance
    QWidget* page = &addPage( IconEngine::get( ICONS::PREFERENCE_APPEARANCE ), tr( "Appearance" ), tr( "Main window appearance and colors" ) );

    page->layout()->addWidget( box = new QGroupBox( tr( "Colors" ),  page ) );
    GridLayout* gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(5);
    gridLayout->setMaxCount(2);
    gridLayout->setColumnAlignment( 0, Qt::AlignRight|Qt::AlignVCenter );
    box->setLayout( gridLayout );

    // colors
    gridLayout->addWidget( new QLabel( tr( "Server application:" ), box ) );
    OptionColorDisplay* color = new OptionColorDisplay( box, "SERVER_COLOR" );
    color->setToolTip( tr( "Item color for server application" ) );
    gridLayout->addWidget( color );
    addOptionWidget( color );

    gridLayout->addWidget( new QLabel( tr( "Identify requests:" ), box ) );
    color = new OptionColorDisplay( box, "IDENTIFY_COLOR" );
    color->setToolTip( tr( "Item color for identification requests" ) );
    gridLayout->addWidget( color );
    addOptionWidget( color );

    page->layout()->addWidget( box = new QGroupBox( tr( "Options" ),  page ) );
    box->setLayout( new QVBoxLayout() );
    box->layout()->setMargin(5);
    box->layout()->setSpacing(5);

    gridLayout = new GridLayout();
    gridLayout->setMargin(0);
    gridLayout->setSpacing(5);
    gridLayout->setMaxCount(2);
    box->layout()->addItem( gridLayout );

    // verbosity
    gridLayout->addWidget( new QLabel( tr( "Verbosity:" ), box ) );
    OptionComboBox* combobox = new OptionComboBox( box, "VERBOSITY" );
    combobox->setUseValue( false );
    combobox->addItem( tr( "low" ) );
    combobox->addItem( tr( "high" ) );
    gridLayout->addWidget( combobox );
    addOptionWidget( combobox );
    combobox->setToolTip( tr( "List display verbosity" ) );

    OptionCheckBox* checkbox;
    checkbox = new OptionCheckBox( tr( "Show application icons when available" ), box, "SHOW_ICONS" );
    addOptionWidget( checkbox );
    box->layout()->addWidget( checkbox );

    SERVER::ServerConfiguration* serverConfiguration;

    // server
    page = &addPage( IconEngine::get( ICONS::PREFERENCE_SERVER ), tr( "Server" ), tr( "Client/server connection to other server-oriented applications" ) );
    page->layout()->addWidget( serverConfiguration = new SERVER::ServerConfiguration( page ));
    addOptionWidget( serverConfiguration );

    // load initial configuration
    read();

}
