#ifndef _MainWindow_h_
#define _MainWindow_h_

// $Id: MainWindow.h,v 1.27 2014/01/16 19:55:27 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "BaseMainWindow.h"
#include "Debug.h"
#include "ServerCommand.h"
#include "ServerCommandModel.h"

#include <QAction>
#include <QCloseEvent>

class CustomToolBar;
class TreeView;
class Menu;
class BaseStatusBar;

/*!
\class MainWindow
\brief customized QVBox to recieve close event
*/
class MainWindow: public BaseMainWindow
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    MainWindow( QWidget* parent = 0 );

    //! verbosity enumeration
    enum Verbosity
    { LOW, HIGH };

    //! Verbosity
    void setVerbosity( Verbosity verbosity )
    { verbosity_ = verbosity; }

    //! menu
    Menu& menu( void )
    { return *menu_; }

    //! statusBar
    BaseStatusBar& statusBar( void )
    { return *statusBar_; }

    //! toolbar
    CustomToolBar& toolBar( void ) const
    { return *toolbar_; }

    //! append command to list
    void append( const ServerCommandModel::List& );

    //!@name actions
    //@{

    //! reset action
    QAction& resetAction( void ) const
    { return *resetAction_; }

    //! unlock action
    QAction& unlockAction( void ) const
    { return *unlockAction_; }

    //! raise action
    QAction& raiseAction( void ) const
    { return *raiseAction_; }

    //! raise action
    QAction& abortAction( void ) const
    { return *abortAction_; }

    //@}

    Q_SIGNALS:

    //! request command to be passed to application application manager
    /*! the application ID corresponds to this application */
    void requestCommand( SERVER::ServerCommand::CommandType );

    //! request command to be passed to application application manager
    void requestCommand( SERVER::ServerCommand );

    protected:

    //! close window event handler
    void closeEvent( QCloseEvent *event );

    private Q_SLOTS:

    //! configuration
    void _updateConfiguration( void );

    //! update actions
    void _updateActions( void );

    //! request reset command
    void _requestReset( void );

    //! request unlock command
    void _requestUnlock( void );

    //! request raise command
    void _requestRaise( void );

    //! request abort command
    void _requestAbort( void );

    private:

    //! actions
    void _installActions( void );

    //! verbosity
    Verbosity verbosity_;

    //! menu bar
    Menu* menu_;

    //! Command list
    TreeView* list_;

    //! command model
    ServerCommandModel model_;

    //! toolbar
    CustomToolBar* toolbar_;

    //! state frame
    BaseStatusBar* statusBar_;

    //!@name actions
    //@{

    //! reset
    QAction* resetAction_;

    //! unlock
    QAction* unlockAction_;

    //! raise application action
    QAction* raiseAction_;

    //! abort application action
    QAction* abortAction_;

    //@}

};

#endif
