// $Id: Menu.cpp,v 1.25 2013/11/15 14:50:40 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Application.h"
#include "CustomToolBar.h"
#include "DebugMenu.h"
#include "Menu.h"
#include "Menu.moc"
#include "MainWindow.h"
#include "Singleton.h"
#include "XmlOptions.h"

//_______________________________________________
Menu::Menu( QWidget* parent ):
    QMenuBar( parent ),
    Counter( "Menu" )
{
    Debug::Throw( "Menu::Menu.\n" );

    // application
    Application& application( *Singleton::get().application<Application>() );

    // top widget
    MainWindow& mainwindow( *static_cast<MainWindow*>( window() ) );

    // generic menu
    QMenu* menu;

    // file menu
    menu = addMenu( tr( "File" ) );
    menu->addAction( &application.closeAction() );

    // create edit menu
    menu = addMenu( tr( "Tools" ) );
    menu->addAction( &mainwindow.resetAction() );
    menu->addAction( &mainwindow.unlockAction() );
    menu->addSeparator();
    menu->addAction( &mainwindow.raiseAction() );
    menu->addAction( &mainwindow.abortAction() );

    // Settings menu
    menu = addMenu( tr( "Settings" ) );

    // verbosity menu
    verbosityMenu_ = menu->addMenu( tr( "Verbosity" ) );
    QActionGroup* verbosity_group = new QActionGroup( menu );
    verbosity_group->setExclusive( true );
    verbosityLow_ = verbosity_group->addAction( tr( "Low" ) );
    verbosityHigh_ = verbosity_group->addAction( tr( "High" ) );
    verbosityLow_->setCheckable( true );
    verbosityHigh_->setCheckable( true );
    verbosityMenu_->addActions( verbosity_group->actions() );

    connect( verbosityMenu_, SIGNAL(aboutToShow()), SLOT(_updateVerbosityMenu()) );
    connect( verbosityMenu_, SIGNAL(triggered(QAction*)), SLOT(_toggleVerbosity(QAction*)) );

    menu->addSeparator();
    menu->addAction( &application.configurationAction() );

    // help menu
    menu = addMenu( tr( "Help" ) );
    menu->addAction( &application.aboutAction() );
    menu->addAction( &application.aboutQtAction() );

    // debug menu
    menu->addSeparator();
    DebugMenu *debugMenu( new DebugMenu( menu ) );
    menu->addAction( debugMenu->menuAction() );
    debugMenu->menuAction()->setVisible( false );

}

//_______________________________________________
void Menu::_updateVerbosityMenu( void )
{
    Debug::Throw( "Menu::_UpdateVerbosityMenu.\n" );

    const int verbosity( XmlOptions::get().get<int>("VERBOSITY") );
    verbosityLow_->setChecked( verbosity == MainWindow::LOW );
    verbosityHigh_->setChecked( verbosity == MainWindow::HIGH );

}

//_______________________________________________
void Menu::_toggleVerbosity( QAction* )
{
    Debug::Throw( "Menu::_toggleVerbosity.\n" );

    XmlOptions::get().set<int>( "VERBOSITY", (verbosityLow_->isChecked()) ? MainWindow::LOW:MainWindow::HIGH );
    XmlOptions::write();

    static_cast<MainWindow*>(window())->setVerbosity( (MainWindow::Verbosity) XmlOptions::get().get<int>( "VERBOSITY" ) );

}
